clc;
clear all;

DataSS = [11.47125    11.4599
             9.916875    12.1365
             7.819235    9.35587
             9.46181     5.20705
             8.59825     1.12474
             10.3827     3.46706
             8.974975    4.87275
             11.79455    16.492
             10.21345    11.2432];
         
DataSS = [DataSS(1,:)
          DataSS(8,:)
          DataSS(7,:)
          DataSS(6,:)
          DataSS(4,:)
          DataSS(3,:)
          DataSS(2,:)
          DataSS(9,:)
          DataSS(5,:)];

DataExp =  [10.520933   10.3338     9.472773    9.713127    10.5659     10.564034   10.4524     7.34347     9.311147
            11.271833   11.154867   11.276533   10.284833   12.194267   10.420633   13.559767   8.087973    11.505433
            11.689033   11.313133   10.390933   9.683377    11.67       10.3458     13.7375     8.965397    12.3316
            11.571967   11.519667   10.177587   9.54622     11.1672     10.031827   13.681466   9.106427    12.531933
            11.425567   11.5337     10.1489     9.358997    11.077733   9.770947    13.5981     9.236267    12.512233
            11.312734   11.465633   10.0525     9.47462     10.948433   9.80332     13.656733   9.329837    12.516367];
Time = [0 12 24 36 48 60]';


NFlox = 1000*0.025;
UV = 0;
y = DataExp;

% Re-arranging Data
y = [y(:,2) y(:,7) y(:,3) y(:,6) y(:,5) y(:,8) y(:,9) y(:,1) y(:,4)];

NewTime = Time(1):3:Time(end);
y = spline(Time,y',NewTime);
y = y';
Time = NewTime';

[nobs,ncol] = size(y);          % Size of Data
nos = 6;


%create basis for N,C,R and B
norder = 4;
knots = linspace(Time(1),Time(nobs),100);
Tlim = max(Time);
nbasis = length(knots) + norder - 2;
basisobj = create_bspline_basis([Time(1),Tlim],[],...
                                norder,knots);

%%setup quadrature points and weights
% Getting intial estimates of coefficients of basis systems
% nquad = 3; %Commented by FA
nquad = 5;   %Added by FA
[basisobj,quadpts,quadwts] = quadset(nquad,basisobj);

%smooth the data with these values
logy = log(y);
lambda1 = 0.1;
fd1Par = fdPar(basisobj,2,lambda1);
[yfd1D2,df1,gcv1] = smooth_basis(Time,y(:,1),fd1Par);

lambda2 = 0.1;
fd2Par = fdPar(basisobj,2,lambda2);
[yfd2D2,df2,gcv2] = smooth_basis(Time,y(:,2),fd2Par);

lambda3 = 0.1;
fd3Par = fdPar(basisobj,2,lambda3);
[yfd3D2,df3,gcv3] = smooth_basis(Time,y(:,3),fd3Par);

lambda4 = 0.1;
fd4Par = fdPar(basisobj,2,lambda4);
[yfd4D2,df4,gcv4] = smooth_basis(Time,y(:,4),fd4Par);

lambda5 = 0.1;
fd5Par = fdPar(basisobj,2,lambda5);
[yfd5D2,df5,gcv5] = smooth_basis(Time,y(:,5),fd5Par);

lambda6 = 0.1;
fd6Par = fdPar(basisobj,2,lambda6);
[yfd6D2,df6,gcv6] = smooth_basis(Time,y(:,6),fd6Par);

lambda7 = 0.1;
fd7Par = fdPar(basisobj,2,lambda7);
[yfd7D2,df7,gcv7] = smooth_basis(Time,y(:,7),fd7Par);

lambda8 = 0.1;
fd8Par = fdPar(basisobj,2,lambda8);
[yfd8D2,df8,gcv8] = smooth_basis(Time,y(:,8),fd8Par);

lambda9 = 0.1;
fd9Par = fdPar(basisobj,2,lambda9);
[yfd9D2,df9,gcv9] = smooth_basis(Time,y(:,9),fd9Par);

%[Ndf, Ngcv;Cdf, Cgcv;Rdf, Rgcv;Bdf, Bgcv];

%Result by smooth data with D2x(t),
%which are the initial value of coef;

coef1 = getcoef(yfd1D2);
coef2 = getcoef(yfd2D2);
coef3 = getcoef(yfd3D2);
coef4 = getcoef(yfd4D2);
coef5 = getcoef(yfd5D2);
coef6 = getcoef(yfd6D2);
coef7 = getcoef(yfd7D2);
coef8 = getcoef(yfd8D2);
coef9 = getcoef(yfd9D2);

coef0 = [coef1;coef2;coef3;coef4;coef5;coef6;coef7;coef8;coef9];

% Weights for each gene concetration
wt = zeros(ncol,1);

for i = 1:9
    wt(i) = var(y(:,i)); 
end
wt = 1./wt;

%basis values at sampling points
basismat = eval_basis(Time,basisobj);
Dbasismat = eval_basis(Time,basisobj,1);

%basis values at quadrature points
quadbasismat = eval_basis(quadpts,basisobj);
size(quadbasismat);
Dquadbasismat = eval_basis(quadpts,...
                basisobj,1);

% Setting Optimization options
optionsPPfn = optimset('LargeScale','on',...
                'DerivativeCheck','off','Display',...
                'off','MaxIter',500,...
                'TolCon',1e-12,'TolFun',1e-6,...
                'TolX',1e-12,...
                'Jacobian','on');
            
% tolval = 1e-6;
options_out = optimset('DerivativeCheck','off',...
                'Jacobian','on','Display','off','MaxIter',...
                1000,'TolFun',1e-6,'TolX',1e-9,...       % 100
                'MaxFunEval',10000);

options_in = optimset('DerivativeCheck','off',...
                'Jacobian','on','Display','off','MaxIter',...
                5000,'TolFun',1e-9,'TolX',1e-9,...       % 100
                'MaxFunEval',10000);
            

Ycell = cell(9,1);
Ycell{1} = y(:,1);
Ycell{2} = y(:,2);
Ycell{3} = y(:,3);
Ycell{4} = y(:,4);
Ycell{5} = y(:,5);
Ycell{6} = y(:,6);
Ycell{7} = y(:,7);
Ycell{8} = y(:,8);
Ycell{9} = y(:,9);

A0 = quadbasismat;
A1 = Dquadbasismat;
A2 = basismat;


%Simpson's rule weights for integration
nknots = length(quadpts);
h = 1/(nknots-1);
W = zeros(nknots,nknots);
W(1,1) = 1/3*h;
W(nknots,nknots) = 1/3*h;
for j = 2:(nknots-1);
    if(mod(j,2)==0)
        W(j,j)=4/3*h;
    else
        W(j,j)=2/3*h;
    end
end

% W = diag(quadwts);
W = W*(Time(end)-Time(1));

% startPar = 0.5*randn((43*1)+(9*2),1);

% Random initial parameters
startPar = [5*(1:9)';3.0*ones(43,1);1.0*ones(43,1);2*(1:9)'];
% startPar = exp(startPar);

% Initial parameters already obtained through varying lambda for fixed
% cooperativity parameters
load betaSpline2;
startPar = betaSp2;
clear betaSp2;
startPar = [startPar(1:52);2.0*ones(43,1);startPar(53:61)];


% getting initial parameters with very little smoothing parameter (lambda)
lambda = (0.01);  
% ub = 0*coef0 + 500;
% lb = 0*coef0 - 100;
temp = @(c)GLSexp(c,startPar,lambda,Ycell,W,A0,A1,A2,wt);
[coefs,resnorm,residual,exitflag] = lsqnonlin(temp,coef0,[],[],optionsPPfn);

beta(:,1) = startPar;

    lam = 1;
    lambda = 0.1;    
    spar = [beta(1:52,lam);beta(96:104,lam)];
    spar = log(spar);    
    npar = beta(53:95,lam);
    npar = log(npar);
    fun = @(par)HLSexp(par,npar,lambda,Ycell,W,A0,A1,A2,...
            coefs,options_in,wt);
    est1par = lsqnonlin(fun,spar,[],[],options_out); 


% Now finding two sets of parameters according to split optimization
% approach (SOT)

for lam=1:10
    % lambda = 10^(lam+1);
%     lambda = 1 + (lam-1)*exp(lam*1.35);
    lambda = 150;

    % ub = [log(50)*ones(9,1);log(50)*ones(43,1);log(5)*ones(43,1);log(50)*ones(9,1)];

    spar = [beta(1:52,lam);beta(96:104,lam)];
    spar = log(spar);    
    npar = beta(53:95,lam);
    npar = log(npar);
    fun = @(par)HLSexp(par,npar,lambda,Ycell,W,A0,A1,A2,...
            coefs,options_in,wt);
    est1par = lsqnonlin(fun,spar,[],[],options_out);

    ub = 5*ones(43,1);
    lb = 0*ones(43,1);
    
    auxpar = exp(est1par);
    npar = exp(npar);
    fun = @(par)HLSexpN(par,auxpar,lambda,Ycell,W,A0,A1,A2,...
            coefs,options_in,wt);
    tbeta = lsqnonlin(fun,npar,lb,ub,options_out);
    beta(:,lam+1) = [auxpar(1:52);tbeta;auxpar(53:61)]; 
    
    
    error = abs(beta(:,lam+1)-beta(:,lam));
    
    if(norm(error,inf) <= 1e-3)
        break;    
    end


end



% Again finding coefficients for parameters at each iteration
% can be skipped if stored previously
for loopiterate = 1:(lam+1)
    
     temp = @(c)GLSexp(c,beta(:,loopiterate),lambda,Ycell,W,A0,A1,A2,wt);
     Coefit(:,loopiterate) = lsqnonlin(temp,coefs,[],[],optionsPPfn);
    
end

% Data again in new variable
ty = DataExp;
ty = [ty(:,2) ty(:,7) ty(:,3) ty(:,6) ty(:,5) ty(:,8) ty(:,9) ty(:,1) ty(:,4)]; 

% Calculating sum of squared errors for set of parameters at each iteration

for loopiterate = 1:(lam+1)
cfit = Coefit(:,loopiterate); 
bfit = beta(:,loopiterate);
mm = length(cfit);

temp1 = A0*cfit(1:(mm/9));
temp2 = A0*cfit((mm/9+1):(2*mm/9));
temp3 = A0*cfit((2*mm/9+1):(3*mm/9));
temp4 = A0*cfit((3*mm/9+1):(4*mm/9));
temp5 = A0*cfit((4*mm/9+1):(5*mm/9));
temp6 = A0*cfit((5*mm/9+1):(6*mm/9));
temp7 = A0*cfit((6*mm/9+1):(7*mm/9));
temp8 = A0*cfit((7*mm/9+1):(8*mm/9));
temp9 = A0*cfit((8*mm/9+1):end);

initial{loopiterate} = [temp1(1),temp2(1),temp3(1),temp4(1),temp5(1),temp6(1),temp7(1),temp8(1),temp9(1)];

odeoptions = odeset('RelTol',1e-10,'AbsTol',1e-10);
Time = [0 12 24 36 48 60]';
tfine = linspace(Time(1),Time(end),20);

[junk,tpro{loopiterate}] = ode45(@SOSMMode,Time,initial{loopiterate},...
                odeoptions,bfit);
[junk,tprofine{loopiterate}]=ode45(@SOSMMode,tfine,initial{loopiterate},...
                odeoptions,bfit);

pro = tpro{loopiterate};             
profine = tprofine{loopiterate};


SSE(loopiterate) = wt(1)*(ty(:,1) - pro(:,1))'*(ty(:,1) - pro(:,1)) + ...
            wt(2)*(ty(:,2) - pro(:,2))'*(ty(:,2) - pro(:,2)) + ...            
            wt(3)*(ty(:,3) - pro(:,3))'*(ty(:,3) - pro(:,3)) + ...            
            wt(4)*(ty(:,4) - pro(:,4))'*(ty(:,4) - pro(:,4)) + ...            
            wt(5)*(ty(:,5) - pro(:,5))'*(ty(:,5) - pro(:,5)) + ...            
            wt(6)*(ty(:,6) - pro(:,6))'*(ty(:,6) - pro(:,6)) + ...            
            wt(7)*(ty(:,7) - pro(:,7))'*(ty(:,7) - pro(:,7)) + ...            
            wt(8)*(ty(:,8) - pro(:,8))'*(ty(:,8) - pro(:,8)) + ...            
            wt(9)*(ty(:,9) - pro(:,9))'*(ty(:,9) - pro(:,9));            
            
        
disp(loopiterate);          

end



% predicting steady state values for parameters of each iteration
% and calculating sum of squared errors in steady state prediction
options_equ = optimset('DerivativeCheck','off',...
                'Jacobian','on','Display','off','MaxIter',...
                5000,'TolFun',1e-12,'TolX',1e-12,...       % 100
                'MaxFunEval',10000);            
initial = [];
xequ = [];  
ssExp = DataSS(:,1);
ssBar = DataSS(:,2);
SSE_Bar = (ssBar - ssExp)'*(ssBar - ssExp);

for loopiterate = 1:(lam+1)
    
    cfit = Coefit(:,loopiterate); 
    bfit = beta(:,loopiterate);
    mm = length(cfit);

    temp1 = A0*cfit(1:(mm/9));
    temp2 = A0*cfit((mm/9+1):(2*mm/9));
    temp3 = A0*cfit((2*mm/9+1):(3*mm/9));
    temp4 = A0*cfit((3*mm/9+1):(4*mm/9));
    temp5 = A0*cfit((4*mm/9+1):(5*mm/9));
    temp6 = A0*cfit((5*mm/9+1):(6*mm/9));
    temp7 = A0*cfit((6*mm/9+1):(7*mm/9));
    temp8 = A0*cfit((7*mm/9+1):(8*mm/9));
    temp9 = A0*cfit((8*mm/9+1):end);

    init(:,loopiterate) = [temp1(1),temp2(1),temp3(1),temp4(1),temp5(1),temp6(1),temp7(1),temp8(1),temp9(1)]';
    
    funtemp = @(x)SOSMModedydx([],x,bfit);
    xequ(:,loopiterate) = fsolve(funtemp,init(:,loopiterate),options_equ);
    
    SSE_SS(loopiterate) = (xequ(:,loopiterate) - ssExp)'*(xequ(:,loopiterate) - ssExp);
    
end


%plotting time series data with simulation using SOT-GPM
close all;
[minVal minInd] = min(SSE);
tEstN = tpro{minInd};
tFixN = tpro{1};
for i = 1:9
    figure;
    plot(Time,ty(:,i),'*k',Time,tEstN(:,i),'k','LineWidth',2);
    legend('Data','Simulation');
end

%plotting data with SOT-GPM results and Baralla's reproduced results
parBaralla = load('Baralla.txt');
[junk,tBar] = ode45(@SOSMModeBaralla,Time,ty(1,:),...
                odeoptions,parBaralla);
for i = 1:9
    figure;
    plot(Time,ty(:,i),'*k',Time,tEstN(:,i),'k',Time,tBar(:,i),'--k','LineWidth',2);
    legend('Data','SOT-GPM','Baralla Results');
end

%plotting all together
figure;
for i = 1:9
    subplot(3,3,i), plot(Time,ty(:,i),'.k',Time,tEstN(:,i),'k',Time,tBar(:,i),'--k','LineWidth',1, 'MarkerSize',15);
end
legend('Data','SOT-GPM','Baralla Results','ORIENTATION','HORIZONTAL');

return;

