function [ Gobj dGobj] = GLSexp(c,par,lambda,Ycell,W,A0,A1,A2,wt)
% This is Objective Function Based on Least Square Function
% named 'G' in Cao-Jia Xu
% 
% c      --  Coeffs
% par    --  parameter Array for ODE
% lambda --  Smoothening Parameter
% Ycell  --  Contains Measurement Data Y
% W      --  Diagonal Matrix of Quad Weights For Simpson's rule
% A0     --  quadbasismat
% A1     --  Dquadbasismat
% A2     --  basismat
% koL      --  Hubber Constant
% wt     --  Weight Associated with each operon

S = size(Ycell,1);
Lt = size(cell2mat(Ycell(1,1)),1);
Ltf = size(A0,1);

Gobj = zeros(S*Lt + S*Ltf,1);

nknots = length(c)/S;


y = zeros(Lt,S);
for i = 1:S
    y(:,i) = cell2mat(Ycell(i,1));
end

esty = zeros(Lt,S);
x = zeros(Ltf,S);
dx = zeros(Ltf,S);
for i = 1:S
    esty(:,i) = A2*c(1+(i-1)*nknots:i*nknots);
    x(:,i) = A0*c(1+(i-1)*nknots:i*nknots);
    dx(:,i) = A1*c(1+(i-1)*nknots:i*nknots);
end


for j = 1:S
    for i = 1:Lt
         Gobj(i+(j-1)*Lt) = sqrt(wt(j))*(y(i,j) - esty(i,j));
    end
end

fx = zeros(Ltf,S);
for i = 1:Ltf
    fx(i,:) = SOSMMode([],x(i,:),par);  
end

for k = 1:S
    for i = 1:Ltf        
        Gobj(i + (k-1)*Ltf + S*Lt) = sqrt(wt(k)*lambda*W(i,i))*...
                                (dx(i,k) - fx(i,k));
    end
end


if nargout > 1
    dGobj = zeros(S*Lt + S*Ltf,size(S*nknots,1)); 
    
    df = zeros(Ltf,S,S);
    for i = 1:Ltf
        df(i,:,:) = dydxSOSMMode([],x(i,:),par,0);      
    end
    
    
    for j = 1:S
        for i = 1:Lt
            dGobj(i+(j-1)*Lt,1+(j-1)*nknots:j*nknots) = -1*...
                    sqrt(wt(j)).*(A2(i,:));   
        end
    end
    
  
    for k = 1:S
        for i = 1:Ltf
            for r = 1:S
                dGobj(i+(k-1)*Ltf+S*Lt,1+(r-1)*nknots:r*nknots) = sqrt(wt(k)*lambda*W(i,i))*...
                    ((k==r).*A1(i,:)  - df(i,k,r).*A0(i,:));                
            end
        end        
    end
    
    dGobj = sparse(dGobj);


    
end




end

