function [Hobj dHobj] = HLSexp(par,npar,lambda,Ycell,W,A0,A1,A2,...
        coefs,options_in,wt)
% This is Objective Vector Valued Function  'H' named in Cao-Jia Xu
% For finding par that mininmize this 'H'
% This Function was not found.
% Detailed explanation goes here
% par      - Set of parameters for ppode
% lambda   - Smoothening Parameter
% Ycell    - A cell array of measurement Data
% W        - Quad Weight Diagonal Matrix 
% A0       - quadbasismat
% A1       - Dquadbasismat
% A2       - basismat
% coefs    - inital value of coefficients
% options_in  -  Options for inner level of optimization i.e for 
%             -  finding cHat for given par
% wt       - Weight Array for each Operon

S = size(Ycell,1);
Lt = size(A2,1);                 % No of measurements
Ltf = size(A0,1);

y = zeros(Lt,S);
for i = 1:S
    y(:,i) = cell2mat(Ycell(i,1));
end

knots = length(coefs)/S;
Hobj = zeros(S*Lt,1);

par = exp(par);
npar = exp(npar);
parLen = length(par);

tpar = [par(1:52);npar;par(53:61)];

temp = @(c)GLSexp(c,tpar,lambda,Ycell,W,A0,A1,A2,wt);
cHat = lsqnonlin(temp,coefs,[],[],options_in);
% save cHat.mat cHat;
% cHat = coefs;

esty = zeros(size(y));
x = zeros(Ltf,S);
for i = 1:S
    esty(:,i) = A2*cHat(1+(i-1)*knots:i*knots);  
    x(:,i) = A0*cHat(1+(i-1)*knots:i*knots);
end

for j = 1:S
    for i = 1:Lt
        Hobj(i+(j-1)*Lt) = sqrt(wt(j))*(y(i,j)-esty(i,j));
    end
end


if nargout > 1
    
    dHobj = zeros(S*Lt,parLen);    
    dHdC = zeros(S*Lt,S*knots);
        
    for i = 1:S
        E = sqrt(wt(1))*eye(Lt);
%         E = sqrt(wt(i))*diag(esty(:,i));  
        dHdC(1+(i-1)*Lt:i*Lt,1+(i-1)*knots:i*knots) = - E*A2;             
    end
    
    d2Gc = d2Gexpdc2(cHat,tpar,lambda,Ycell,W,A0,A1,A2,wt);
    d2Gdcp = d2Gexpdcdp(cHat,par,npar,lambda,Ycell,W,A0,A1,wt);
    
    dHobj = dHdC*(-1*d2Gc\d2Gdcp);
    dHobj = sparse(dHobj);

end



end

